package com.filenet.wcm.toolkit.server.operations.contentmanager;

import com.filenet.wcm.api.*;
import com.filenet.wcm.toolkit.server.operations.util.CIConfigProperties;
import filenet.vw.api.*;

import java.io.*;
import java.util.Date;
//import java.util.List;

public class CEDocument
{
	private static CIConfigProperties conf = null;
    	private static String userId   = "Administrator";
    	private static String password = "password";
    	private static String appId    = "CEDocument";

    	private static ObjectStore docObjectStore = null;
    	private static Folder docFolder = null;
    	private static Session sess = null;
    	private static boolean error = false;
    	private static Document d = null;
    	private static Documents dc = null;
    	private static VWAttachment vwa = null;
    	private static Search mySearch = null;

	private static void log(String message)
	{
		String logfilename = "C:\\Integrator.log";
		
		try {
			if (conf==null)	conf = new CIConfigProperties();
			logfilename = conf.getValue("logfilename",logfilename);
			FileWriter file = new FileWriter(logfilename, true);
			BufferedWriter w = new BufferedWriter(file);
			w.write(new Date() + ": ");
			w.write(message);
			w.newLine();
			w.close();
		} catch (Exception e) {}
	}
	    
    	public static Document newDoc(String objStore, String docClassName, String folderName, String filePath,
                              String mimeType, String documentTitle)
    	{
		log("CEDocument_newDoc");
        	initialize(objStore, folderName);
	  	if (!error)
        		d = createTheDocument(documentTitle, docClassName);
	  	if (!error)
        		addContent(d, mimeType, filePath);
	  	if (!error)
        		file(d);
	  	if (!error)
	  	{
	  		log ("OK");
	  		return d;
	  	}
	  	else
	  	{
	  		return null;
	  	}
    	}

    	public static String sDoc(String objStore, String docClassName, String propertyName, String propertyValue)
    	{
		try
	  	{
	  		log("CEDocument_sDoc");
      			initialize(objStore, "/");
		
   			String searchStatement = "<?xml version=\"1.0\" ?>"+
      						 "<request>"+
        					 "<objectstores mergeoption=\"none\">"+
        					 "<objectstore id=\""+objStore+"\" />"+
        					 "</objectstores>"+
        					 "<querystatement>SELECT Id FROM "+docClassName+" WHERE "+
        					 propertyName+"="+propertyValue+"</querystatement>"+
        					 "<options maxrecords=\"100\" />"+
      						 "</request>";
   			log ("Search Statement: "+searchStatement);
	  		if (!error)
	  		{
	  			dc = (Documents)mySearch.singleObjectTypeExecute(searchStatement, BaseObject.TYPE_DOCUMENT);
	  			if (!dc.isEmpty())
	  			{
	  				d = (Document)dc.get(0);
	  	    			log ("document(s) found, first id: " + d.getId());
	  			}
	  			else
	  			{
	  				log("no document found.");
	  			}
	  		}
	  	}
	  	catch (Exception e)
	  	{
	  		log("Error: "+e);
	  		return "";
	  	}
	  	return d.getId();	  
    	}
    
    	private static void initialize(String objStore, String folderName)
    	{
      		userId = conf.getValue("ce_user", userId);
      		password = conf.getValue("ce_password", password);
      		sess = ObjectFactory.getSession(appId, null, userId, password);
	  	if (sess == null)
	  	{
	    		error=true;
	    		log("No session object got!");
	    		return;
	  	}
	  	log("Session object okay");
	  	if (sess.verify() == null)
	  	{
	    		error=true;
	    		log("No user object got or wrong credentials!");
	    		return;
	  	}
	  	log("User object okay");
      		objStore = conf.getValue("ce_objStore", objStore);
	  	docObjectStore = ObjectFactory.getObjectStore(objStore, sess);
	  	if (docObjectStore == null)
	  	{
	    		error=true;
	    		log("No object store object got!");
	    		return;
	  	}
	  	log("Object store object okay");
        	docFolder = (Folder)docObjectStore.getObject(BaseObject.TYPE_FOLDER, folderName);
	  	if (sess == null)
	  	{
	    		error=true;
	    		log("No folder object got!");
	    		return;
	  	}
	  	mySearch = ObjectFactory.getSearch(sess);
	  	if (mySearch == null)
	  	{
	    		error=true;
	    		log("No search object got!");
	    		return;
	  	}	  
	  	log("Search object okay");
	  	log("Initialze okay");
    	}

	private static Document createTheDocument(String documentTitle, String docClassName)
    	{
        	com.filenet.wcm.api.Properties ps = ObjectFactory.getProperties();
	  	if (ps == null)
	  	{
	    		error=true;
	    		log("No properties object got!");
	    		return null;
	  	}
	  	log("Properties object okay");
        	Property p = ObjectFactory.getProperty(Property.DOCUMENT_TITLE);
	  	if (p == null)
	  	{
	    		error=true;
	    		log("No property object got!");
	    		return null;
	  	}
	  	log("Property object okay");
        	p.setValue(documentTitle);
        	ps.add(p);
        	Document d = (Document)docObjectStore.createObject(docClassName, ps, null);
	  	if (d == null)
	  	{
	    		error=true;
	    		log("No document object got!");
	    		return null;
	  	}
	  	log("Create document okay");
        	return d;
    	}

    	private static void addContent(Document d, String mimeType, String filePath)
    	{
		TransportInputStream tis;
		try
		{
        		tis = new TransportInputStream(new FileInputStream(filePath));
		}
		catch (Exception e)
		{
	  		error=true;
	  		log("TransportInputStream-Error: "+e);
	  		return;
		}
      		tis.setMimeType(mimeType);
      		tis.setFilename(filePath);
      		d.setContent(tis, true, false);
	  	log("Add Content okay");
    	}

    	private static void file(Document d)
    	{
        	d.file(docFolder, true);
	    	log("File document okay");
    	}

	/*public static void main(String[] args)
	{
		//newDoc("DMS", "Document", "/Kundenakten", "C:\\Test.pdf", "application/pdf", "test.pdf");
		//sDoc("eFormsPTB", "Banf", "RequestNo", "32");
		System.out.println("fertig");
	}*/
    
}
