/*
 *{{==============================================================================
 *    Copyright (c) 2002, 2003 FileNet Corporation. All rights reserved.
 *
 *    FileNet Proprietary and confidential - Please refer to the Solution Template
 *    Schedule and the readme file for Solution Template Usage Terms.
 *}}==============================================================================
 *    Workfile:   MailIntegrator.java
 *    Revision:   
 *    Date:   27.06.200321:44:09
 */
 
package com.filenet.wcm.toolkit.server.operations.mail;

/**
 * @author rbohnsack
 * 
 */

import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.DataHandler;
import com.filenet.wcm.toolkit.server.operations.util.*;
import filenet.vw.api.VWAttachment;

public class MailIntegrator {

	private static CIConfigProperties conf = null;
	private static final String INBOX 	= "Inbox", POP_MAIL="pop3", SMTP_MAIL="smtp";

	/**
	 * Generates a entry in the log file
	 * @param message The message that will be logged
	 */
	private static void log(String message)
	{
		String logfilename = "C:\\Integrator.log";
		
		try {
			if (conf==null)	conf = new CIConfigProperties();
			logfilename = conf.getValue("logfilename",logfilename);
			FileWriter file = new FileWriter(logfilename, true);
			BufferedWriter w = new BufferedWriter(file);
			w.write(new Date() + ": ");
			w.write(message);
			w.newLine();
			w.close();
		} catch (Exception e) {}
	}
	
	private static String replaceValues(String value, String[] pattern, String[] replace)
	{
		String result = value;
		if (pattern==null || replace==null || value==null)
			return "";
		if (pattern.length>0)
		{
			for (int i=0;(i<pattern.length) && (i<replace.length);i++)
			{	
				result = replacePattern(result,"#" + pattern[i] + "#",replace[i]);
			}
		}
		Date d = new Date();
		result = replacePattern(result,"#DATE#",d.toString());
		return result;
	}
	
	private static String replacePattern(String value, String pattern, String replace)
	{
		String result = value;		
		int index = -1; 
		if (pattern==null || replace==null || value==null)
			return value;
		while ( (index = result.indexOf(pattern.toString())) > -1 ) { 
			result = result.substring( 0, index ) + replace + result.substring( index + pattern.length(), result.length() ); 
		}
		return result; 
	}

	private static void setMailContent(String in, Part msg, String type) throws MessagingException
	{
		msg.setDataHandler(new DataHandler(new ByteArrayDataSource(in, type)));
	}

	private static void createMailAttachment(Multipart mmp, VWAttachment att) throws MessagingException
	{	
		MimeBodyPart mbp = new MimeBodyPart();
		String[] s =  CEUtils.getVWContent(att);
		setMailContent(s[0],mbp,s[1]);
		mbp.setDisposition(Part.ATTACHMENT);
		mmp.addBodyPart(mbp);
	}

	/**
	 * Sends a textmail with the specified message and subject to a mailadress
	 * The sender adress and the mailserver adress comes from the configuration file
	 * 
	 * @param mail_adress String Mailadress of the receiver
	 * @param mail_subject String Subject of the mail
	 * @param mail_message String Mailcontent
	 * @return String Errormessage
	 */
	public static String sendTextMail(String mail_adress, 
									  String mail_subject, 
									  String mail_message)
	{		 
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		try
		{
			if (conf==null)	conf = new CIConfigProperties();
			String pop3Host = conf.getValue("mail_smtphost");
			String from = conf.getValue("mail_from");
			java.util.Properties props = new java.util.Properties();
			props.put("mail.smtp.host", pop3Host);
			props.put("mail.transport.protocol", SMTP_MAIL);
	
			javax.mail.Session session = javax.mail.Session.getDefaultInstance(props, null);
			Message msg = new MimeMessage(session);
			if (from != null)
				msg.setFrom(new InternetAddress(from));
			else
				msg.setFrom();
			InternetAddress[] address = {new InternetAddress(mail_adress)};
			msg.setRecipients(Message.RecipientType.TO, address);
			msg.setSubject(mail_subject);
			msg.setSentDate(new Date());
			msg.setText(mail_message);
			log("Send mail to" + mail_adress + " with subject " + mail_subject + " from " + from);
			Transport.send(msg);
			return "";
		}
		catch (MessagingException me)
		{
			me.printStackTrace(pw);
			log(sw.toString());  
			log("Messageerror: " + me.getMessage()); 
			return "Messageerror: " + me.getMessage();
		}
		catch (Exception e)
		{
			e.printStackTrace(pw);
			log(sw.toString());  
			log("Error " + e.getMessage());
			return "Error " + e.getMessage();
		}
	}

	/**
	 * Receives a mail from the inbox of the described user on a mailserver.
	 * The mailserver adress comes from the configuration file. 
	 *  
	 * @param user String Username
	 * @param password String Password
	 * @return String[0] Replyadress
	 * 		   String[1] Subject
	 * 		   String[2] Senddate
	 * 		   String[3] Message
	 */
	public static String[] receiveTextMail(String user, String password)
	{
		String[] emptyresult = {"","","",""};
		String[] result = new String[4];
		Message recMessage = null;
		String replyTo = "";
		Address[] reply = null;
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		
		try
		{
			if (conf==null)	conf = new CIConfigProperties();
			String pop3Host = conf.getValue("mail_pop3host");
			java.util.Properties sysProperties = System.getProperties();
			sysProperties.put("mail.host", pop3Host);
			sysProperties.put("mail.store.protocol", POP_MAIL);
			javax.mail.Session session = javax.mail.Session.getDefaultInstance(sysProperties, null);
			
			// Connect to host
			Store store = session.getStore(POP_MAIL);
			store.connect(pop3Host, -1, user, password);   

			// Open the default folder
			javax.mail.Folder folder = store.getDefaultFolder();
			if (folder == null)
				throw new NullPointerException("No default mail folder");

			folder = folder.getFolder(INBOX);
			if (folder == null)
				throw new NullPointerException("Unable to get folder: " + folder);

			// Get message count
			folder.open(javax.mail.Folder.READ_WRITE);
			int totalMessages = folder.getMessageCount();
			if (totalMessages == 0)
			{
				log("Folder Inbox is empty.");
				folder.close(false);
				store.close();
				return emptyresult;
			}
    
			Message[] messages = folder.getMessages();
			FetchProfile fp = new FetchProfile();
			fp.add(FetchProfile.Item.ENVELOPE);
			fp.add(FetchProfile.Item.FLAGS);
			fp.add("X-Mailer");
			folder.fetch(messages, fp);
			recMessage = messages[0];
			
			if (recMessage != null)
			{
				if ((reply = recMessage.getFrom()) != null)
					for (int i=0;i<reply.length;i++)
					{
						if (replyTo.length()==0)
							replyTo = reply[i].toString();
						else
							replyTo += ";"+reply[i].toString();
					}

				result[0] = replyTo;
				result[1] = recMessage.getSubject();
				result[2] = recMessage.getSentDate().toString();
				ByteArrayOutputStream op = new ByteArrayOutputStream();
				Object content = recMessage.getContent();
				if (content instanceof Multipart)
				{
					Multipart mpContent=(Multipart)content;
					result[3]=(String)mpContent.getBodyPart(0).getContent();
				}
				else
				{
					result[3]=(String)recMessage.getContent();
				}
				recMessage.setFlag(Flags.Flag.DELETED, true); 
			}
			folder.close(true);
			store.close();
		
			return result;
		}
		catch (Exception e)
		{
			e.printStackTrace(pw);
			log(sw.toString());  
			log(e.getMessage());
			result[0]="Error";
			result[1]=e.getMessage();
			return result;
		}
	}
	/**
	 * The same functionality as receiveTextMail, but the username and the password
	 * comes also from the configuration file.
	 * 
	 * @return String[0] Replyadress
	 * 		   String[1] Subject
	 * 		   String[2] Senddate
	 * 		   String[3] Message
	 * @throws Exception
	 */
	public static String[] receiveDefaultTextMail() throws Exception 
	{
		if (conf==null)	conf = new CIConfigProperties();
		return receiveTextMail(conf.getValue("mail_user"), conf.getValue("mail_pass"));
	}

	/**
	 * Sends a textmail with the specified message and subject to mailadresses
	 * and CC. The values will replace the mark in the template file. To create a mark
	 * use the syntax #name#. The attachments will be send also with the mail.
	 * 
	 * @param template VWAttachment Template for the mail
	 * @param atts VWAttachment[] Attachments for the mail
	 * @param sendTo String TO adresses ; separated
	 * @param subject String Subject for the mail
	 * @param sendCC CC adresses ; separated
	 * @param templateNames String[] Names of the marks
	 * @param templateValues String[] Corresponding values for the names
	 * @param isHTML boolean Send the mail as HTML
	 * @return String Normally empty, if not - error message
	 */
	public static String sendReplaceMail(VWAttachment template,
										 VWAttachment[] atts,
										 String sendTo,
										 String subject,
										 String sendCC,
										 String[] templateNames,
										 String[] templateValues,
										 boolean isHTML)
	{	
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		String mimeType = "text/plain";
		
		try
		{
			if (conf==null)	conf = new CIConfigProperties();
			String pop3Host = conf.getValue("mail_smtphost");
			String from = conf.getValue("mail_from");
			java.util.Properties props = new java.util.Properties();
			props.put("mail.smtp.host", pop3Host);
			props.put("mail.transport.protocol", SMTP_MAIL);
	
			javax.mail.Session session = javax.mail.Session.getDefaultInstance(props, null);

			Message msg = new MimeMessage(session);
			if (from != null)
				msg.setFrom(new InternetAddress(from));
			else
				msg.setFrom();
			
			if (sendTo==null)
				return "Error sendTo";
			
			msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(sendTo, false));
			if (sendCC!=null)
				msg.setRecipients(Message.RecipientType.CC, InternetAddress.parse(sendCC, false));
			msg.setSubject(subject);
			msg.setSentDate(new Date());
			if (isHTML)
			{
				msg.setHeader("X-Mailer", "sendhtml");
				mimeType="text/html";
			}

			String[] s =  CEUtils.getVWContent(template);
			String content = replaceValues(s[0],templateNames,templateValues);
			mimeType = s[1];
			
			if (atts.length>0)
			{
				MimeBodyPart mbp1 = new MimeBodyPart();
				mbp1.setText(content);
				setMailContent(content, mbp1, mimeType);
				Multipart mp = new MimeMultipart();
				mp.addBodyPart(mbp1);
				for (int i=0;i<atts.length;i++)
				{
					createMailAttachment(mp,atts[i]);
				}
				msg.setContent(mp);
			}
			else
				setMailContent(content, msg, mimeType);
				
			log("Send mail to" + sendTo + " with subject " + subject + " from " + from);
			
			Transport.send(msg);
			return "";
		}
		catch (Exception e)
		{
			e.printStackTrace(pw);
			log(sw.toString());  
			log(e.getMessage());
			return e.getMessage();
		}
	}
	
	
	/*public static void main(String[] args)
	{
		String test1 = sendTextMail("kzimmermann@web.de","Testmail","Dies ist ein Test");
		System.out.println("test1 = " + test1);
		String[] test = receiveTextMail("kzimmermann","password");
		System.out.println(test[0]);
		System.out.println(test[1]);
		System.out.println(test[2]);
		System.out.println(test[3]);	
	}*/
	
}


