/*
 *{{==============================================================================
 *    Copyright (c) 2002, 2003 FileNet Corporation. All rights reserved.
 *
 *    FileNet Proprietary and confidential - Please refer to the Solution Template
 *    Schedule and the readme file for Solution Template Usage Terms.
 *}}==============================================================================
 *    $Workfile:   CIConfigProperties.java  $
 *    $Revision:   1.3  $
 *    $Date:   11 Feb 2003 12:54:10  $
 */

package com.filenet.wcm.toolkit.server.operations.util;

import java.io.*;
import java.util.*;

/**
 * Utility that can be used to read in configuration property file.
 */
public class CIConfigProperties
{
	/**
	 * The full computed path to the property file.
	 */
	protected String propertiesPath = null;
	/**
	 * The property resource bundle.
	 */
	protected PropertyResourceBundle prb = null;

	/**
	 * Generates a entry in the log file
	 * @param message The message that will be logged
	 */
	private void log(String message)
	{
		try {
				String logfilename = "C:\\CIConfigProperties.log";
				FileWriter file = new FileWriter(getValue("logfilename",logfilename), true);
				BufferedWriter w = new BufferedWriter(file);
				w.write(new Date() + ": ");
				w.write(message);
				w.newLine();
				w.close();
		} catch (IOException ex) {}
	}

	/**
	 * Checks if the file with the given filename exists.
	 * @param name Filename to check
	 * @return true if the file exists
	 */
	private boolean CheckFileExists(String name)
	{
		File f = new File(name);
		return f.exists();
	}

	/**
	 * @param name Name if the entry in the classpath
	 * @return Generated filename
	 */
	private String BuildFileName(String name)
	{
		String result = "";
		File f = new File(name);
		if (f.isDirectory())
		{
			 result = name;
		}
		else
			result = f.getParent();
		result = result + File.separator + "CI3.properties";
		if (CheckFileExists(result))
		{ 
			return result;
		}
		else
			return "";
	}

	/**
	 * Produce and check the filename for the configuration file from the classpath
	 * 
	 * @return Produced filename
	 */
	private String GetFileName()
	{
		String s = System.getProperty("java.class.path");
		System.out.println ("java path = "+s);
		log ("java path = "+s);
		StringTokenizer st = new StringTokenizer(s,";");
		while (st.hasMoreTokens()) 
		{
			s = BuildFileName(st.nextToken());
			if (s!="") return s;
			
		}
		return "";
	}
   
    /**
     * Reads in the configuration property file from [classpath]\CI3.properties.
     */
    public CIConfigProperties() throws Exception
    {
        if (prb == null)
        {
            //propertiesPath = GetFileName();
            propertiesPath = "C:\\Components\\CI3.properties";
            FileInputStream input = null;
            try
            {
                input = new FileInputStream(propertiesPath);
                prb = new PropertyResourceBundle(input);
            }
            finally
            {
                if (input != null)
                    input.close();
            }
        }
    }

    /**
     * Returns the configuration setting based on the key.
     */
    public String getValue(String key)
    {
        // If the config property file is missing the key speified, a java.util.MissingResourceException will be thrown.
        return prb.getString(key);
    }

	/**
	 * Returns the configuration setting based on the key.
	 * If the key not exists it returns the default value.
	 * 
	 * @param key Key for selectin a value
	 * @param defaultvalue This value will be returned, if the value for the given key not exists
	 * @return Value for the key or defaultvalue
	 */
	public String getValue(String key, String defaultvalue)
	{
		String value="";
		try
		{
			value = getValue(key);
		}
		catch (MissingResourceException ex)
		{
			value = defaultvalue;
		}
		return value;
	}

    /**
     * Gets the keys of all the properties in the config file.
     */
    public Enumeration getKeys()
    {
        Enumeration keys = null;

        if (prb != null)
            keys = prb.getKeys();

        return keys;
    }
}
